import java.util.*;

public class OrderMatching {

	private List<SellOrder> sellOrderQueue = new LinkedList<>();
	private List<BuyOrder> buyOrderQueue = new LinkedList<>();
	
	/*
	 * Checks if the order is valid
	 */
	boolean validate(Order order) {		
		return order.getQty() > 0 && order.getPrice() > 0.0;
	}
	
	/*
	 * General method to handle both BuyOrder and SellOrder
	 */
	ArrayList<Order> findMatchedOrders(Order o) {
		return (o.getType() == "SELL") 
						? findBuyOrders((SellOrder) o) 
						: findSellOrders((BuyOrder) o);
	}
	/*
	 * Checks if trade can happen between buy order and
	 * sell order
	 */
	boolean buySellOrderMatch(BuyOrder b, SellOrder s) {
		return !b.getCustomer().equals(s.getCustomer()) &&
				b.getStock().equals(s.getStock()) &&
				b.getPrice() >= s.getPrice();
	}
	
	/*
	 * Finds all buy orders for current sell order
	 */
	ArrayList<Order> findBuyOrders(SellOrder s) {
		ArrayList<Order> matchedOrders = new ArrayList<>();
		
		// Iterate over buy orders queue
		for (BuyOrder b : buyOrderQueue) {
			// if current sell orders have already found all buyers,
			// exit the loop
			if (s.getQty() <= 0) break;
			
			// check if trade is possible
			if (buySellOrderMatch(b, s)) {
				
				int qty = s.getQty() - b.getQty();
				
				if (qty >= 0) {				
					// modify sell order quantity to prevent 
					// repeated purchases
					s.setQty(qty);
					matchedOrders.add(b);
				} else {				
					// create a temporary BuyOrder with quantity equal to 
					// the remaining stocks in sell order
					BuyOrder _b = new BuyOrder(b);
					_b.setQty(s.getQty());
					// add it to buy orders queue
					matchedOrders.add(_b);

					s.setQty(0);

					// modify buy order's quantity to reflect quantity
					// remaining after trade
					b.setQty(b.getQty() - _b.getQty());
				}
			}	
		}
		
		// if there is still some stocks to sell
		// add it to sell orders queue
		if (s.getQty() > 0) sellOrderQueue.add(s);
		
		// remove all buy orders that were matched from
		// buy order queue
		for (Order b : matchedOrders) {
			buyOrderQueue.remove(b);
		}
		
		return matchedOrders;
	}
	
	/*
	 * Finds all sell orders for current buy order
	 */
	ArrayList<Order> findSellOrders(BuyOrder b) {
		ArrayList<Order> matchedOrders = new ArrayList<>();
		
		// Iterate over sell orders queue
		for (SellOrder s : sellOrderQueue) {
			// if current buy order has already found all sellers,
			// exit the loop
			if (b.getQty() <= 0) break;
			
			// check if trade is possible
			if (buySellOrderMatch(b, s)) {
				
				int qty = b.getQty() - s.getQty();
				
				if (qty >= 0) {				
					// modify buy order quantity to prevent 
					// repeated purchases
					b.setQty(qty);
					matchedOrders.add(s);
				} else {
					// create a temporary SellOrder with quantity equal to 
					// the remaining stocks in buy order
					SellOrder _s = new SellOrder(s);
					_s.setQty(b.getQty());

					// add it to matched orders queue
					matchedOrders.add(_s);
					b.setQty(0);

					// modify sell order's quantity to reflect quantity
					// remaining after trade
					s.setQty(s.getQty() - _s.getQty());
				}
			}	
		}
		
		// if there is still some stocks to buy
		// add it to buy orders queue
		if (b.getQty() > 0) buyOrderQueue.add(b);
		
		// remove all sell orders that were matched from
		// sell order queue
		for (Order s : matchedOrders) {
			sellOrderQueue.remove(s);
		}
		
		return matchedOrders;
	}
	
	List<SellOrder> printSellOrderQueue() {
		return sellOrderQueue;
	}
	
	List<BuyOrder> printBuyOrderQueue() {
		return buyOrderQueue;
	}
}
