import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.Test;

class OrderMatchingTest {

	@Test
	void buySellOrderMatchTest() {
		Customer c1 = new Customer("c1");
		Customer c2 = new Customer("c2");
		Stock appleStock = new Stock("apple");
		Stock googleStock = new Stock("google");
		
		OrderMatching orderMatching = new OrderMatching();
		
		BuyOrder b1 = new BuyOrder(1, c1, googleStock, 100, 300);
		BuyOrder b2 = new BuyOrder(2, c2, googleStock, 300, 200);
		BuyOrder b3 = new BuyOrder(5, c1, googleStock, 300, 200);
		SellOrder s1 = new SellOrder(3, c2, googleStock, 200, 200);
		SellOrder s2 = new SellOrder(4, c2, appleStock, 200, 200);
		
		assertTrue(orderMatching.buySellOrderMatch(b1, s1));
		assertFalse(orderMatching.buySellOrderMatch(b1, s2));
		assertFalse(orderMatching.buySellOrderMatch(b1, s2));
		assertFalse(orderMatching.buySellOrderMatch(b2, s1));
		assertTrue(orderMatching.buySellOrderMatch(b3, s1));
	}
	
	@Test
	void findOrdersTest() {
		Customer c1 = new Customer("c1");
		Customer c2 = new Customer("c2");
		Customer c3 = new Customer("c3");
		Customer c4 = new Customer("c4");
		
		Stock appleStock = new Stock("apple");
		
		OrderMatching orderMatching = new OrderMatching();
		
		SellOrder s1 = new SellOrder(10, c1, appleStock, 25, 325);
		SellOrder s2 = new SellOrder(12, c2, appleStock, 30, 320);
		BuyOrder b1 = new BuyOrder(13, c3, appleStock, 20, 315);
		BuyOrder b2 = new BuyOrder(15, c4, appleStock, 35, 330);
		
		assertEquals(orderMatching.findBuyOrders(s1).size(), 0);
		assertEquals(orderMatching.findBuyOrders(s2).size(), 0);
		assertEquals(orderMatching.findSellOrders(b1).size(), 0);
		assertNotEquals(orderMatching.findSellOrders(b2).size(), 0);
		
		System.out.println(orderMatching.printBuyOrderQueue());
		System.out.println(orderMatching.printSellOrderQueue());
	}

	@Test
	void validateOrderTest() {
		Customer c1 = new Customer("c1");
		Customer c2 = new Customer("c2");
		Stock appleStock = new Stock("apple");
		Stock googleStock = new Stock("google");
		
		OrderMatching orderMatching = new OrderMatching();
		
		BuyOrder b1 = new BuyOrder(1, c1, googleStock, 100, 300);
		BuyOrder b2 = new BuyOrder(2, c2, googleStock, 300, 200);
		BuyOrder b3 = new BuyOrder(5, c1, googleStock, -300, 200);
		SellOrder s1 = new SellOrder(3, c2, googleStock, 200, 200);
		SellOrder s2 = new SellOrder(4, c2, appleStock, 200, -200);
		
		assertTrue(orderMatching.validate(b1));
		assertTrue(orderMatching.validate(b2));
		assertFalse(orderMatching.validate(b3));
		assertTrue(orderMatching.validate(s1));
		assertFalse(orderMatching.validate(s2));
	}
}
